process.traceDeprecation = true;
const webpack = require('webpack');
const path = require('path');
const BundleAnalyzerPlugin = require('webpack-bundle-analyzer').BundleAnalyzerPlugin;

// load the babel.config.js from this library and pass that into babel-loader
// as otherwise it won't find it and then the transformation will fail
// UNLESS you'd copied the babel.config.js to your local folder.
const babelConfig = require('./babel.config.js');

const debug = true;
const webpackMode = debug ? 'development' : 'production';

const arrPlugins = [];
arrPlugins.push(new webpack.DefinePlugin({
  'process.env.NODE_ENV': JSON.stringify(webpackMode)
}));
if (!debug) {
  /* generage bundle analyzer for production */
  arrPlugins.push(new BundleAnalyzerPlugin({ analyzerMode: 'static' }));
}

module.exports = {
  target: ['web', 'es5'], // even though Babel's configured for ie11, target Webpack for ES5 too, so it doesn't use arrow functions itself
  resolve: {
    extensions: ['.ts', '.js', '.json']
  },
  /* see https://webpack.js.org/configuration/devtool/ */
  // devtool: debug ? 'eval-source-map' : 'cheap-source-map',
  devtool: 'source-map',
  /* https://webpack.js.org/configuration/entry-context/#context */
  context: path.resolve(__dirname, ''),
  mode: webpackMode,
  /* see https://webpack.js.org/configuration/entry-context/#entry */
  entry: {
  },
  /* see https://webpack.js.org/configuration/plugins/#plugins */
  plugins: arrPlugins,
  /* see https://webpack.js.org/configuration/output/ */
  output: {
    filename: '[name].js',
    path: '', // <-- to be provided by caller
    pathinfo: debug,
    // enforce: 'pre' /* no idea if/why this is needed. */
  },
  /* see https://webpack.js.org/configuration/module/ */
  module: {
    rules: [
      {
        test: /\.ts$/,
        exclude: [/node_modules/],
        /* these loaders are executed last to first order */
        use: {
          loader: 'babel-loader',
          options: babelConfig
          // 'eslint-loader',
        }
      },
      {
        test: /\.js$/,
        exclude: [/node_modules\/(?!(debug)\/).*/], // exclude node_modules EXCEPT the debug module, which we need to transpile
        /* these loaders are executed last to first order */
        use: {
          loader: 'babel-loader',
          options: babelConfig
          // 'eslint-loader',
        }
      },
      {
        /* less files will be parsed to css then embedded into js files */
        test: /\.less$/,
        /* order in which loaders are used is right to left (last to first) */
        use: ['style-loader', 'css-loader', 'less-loader']
      },
      {
        /* use html-loader for *.htm files */
        test: /\.htm$/,
        /* order in which loaders are used is right to left (last to first) */
        use: ['html-loader']
      },
      {
        /* use file-loader to ensure all matching file types are copied without change to output folder */
        test: /\.(html|jpe?g|gif|png|ico|svg|woff|ttf|wav|mp3|eot|woff2)$/,
        /* order in which loaders are used is right to left (last to first) */
        use: ['file-loader?name=[name].[ext]'] /* don't use [path] to keep output directory flat */
      },
      {
        /* css files will be embedded into js files */
        test: /\.css$/,
        /* order in which loaders are used is right to left (last to first) */
        use: ['style-loader', 'css-loader']
      }
    ]
  }
};
